@@ -85,7 +85,7 @@ typedef struct{
 	int veraCrypt_volume ;
 	int trueCrypt_volume ;
 	int bitlocker_volume ;
-	int tcrypt_system ;
+	int system_volume ;
 	const char * const * tcrypt_keyfiles ;
 	unsigned long m_flags ;
 	int use_backup_header ;
@@ -137,6 +137,11 @@ static inline int zuluCryptWhirlpoolIsSupported()
  */
 const char * zuluCryptBitLockerType( void ) ;
 
+/*
+ * zuluCryptVeraCryptPIM() is defined in volume_type.c
+ */
+const char * zuluCryptBitLockerFolderPrefix( void ) ;
+
 /*
  * zuluCryptVeraCryptPIM() is defined in create_tcrypt.c
  */
