@@ -41,12 +41,15 @@
 #include "task.hpp"
 #include "dialogmsg.h"
 #include "plugin.h"
+#include "utility.h"
 
 luksaddkey::luksaddkey( QWidget * parent ) : QDialog( parent )
 {
 	m_ui = new Ui::luksaddkey() ;
 	m_ui->setupUi( this ) ;
 
+	m_label.setOptions( m_ui->label_5,m_ui->pushButton ) ;
+
 	m_isWindowClosable = true ;
 
 	m_ui->textEditPathToVolume->setText( QString() ) ;
@@ -181,6 +184,8 @@ void luksaddkey::cbVolumeType( int e )
 
 void luksaddkey::cbExistingKey( int e )
 {
+	m_yubikeyExistingKey = false ;
+
 	auto _key_ui = [ this ](){
 
 		m_ui->textEditExistingPassphrase->setToolTip( tr( "Enter A Key" ) ) ;
@@ -207,7 +212,9 @@ void luksaddkey::cbExistingKey( int e )
 		m_ui->pushButtonOpenExistingKeyFile->setIcon( QIcon( ":/keyfile.png" ) ) ;
 		m_ui->textEditExistingPassphrase->setFocus() ;
 		m_ui->textEditExistingPassphrase->setEnabled( true ) ;
-	}else{
+
+	}else if( e == 2 ){
+
 		_key_ui() ;
 
 		m_ui->textEditExistingPassphrase->setEnabled( false ) ;
@@ -222,11 +229,16 @@ void luksaddkey::cbExistingKey( int e )
 				this->cbExistingKey( 0 ) ;
 			}
 		} ) ;
+	}else{
+		_key_ui() ;
+		m_yubikeyExistingKey = true ;
 	}
 }
 
 void luksaddkey::cbNewKey( int e )
 {
+	m_yubekeyNewKey = false ;
+
 	auto _key_ui = [ this ](){
 
 		m_ui->textEditPassphraseToAdd->setToolTip( tr( "Enter a key" ) ) ;
@@ -259,7 +271,9 @@ void luksaddkey::cbNewKey( int e )
 		m_ui->pushButtonOpenNewKeyFile->setIcon( QIcon( ":/keyfile.png" ) ) ;
 		m_ui->textEditPassphraseToAdd->setFocus() ;
 		m_ui->textEditPassphraseToAdd->setEnabled( true ) ;
-	}else{
+
+	}else if( e == 2 ){
+
 		_key_ui() ;
 
 		m_ui->textEditPassphraseToAdd->setEnabled( false ) ;
@@ -281,6 +295,12 @@ void luksaddkey::cbNewKey( int e )
 				}
 			}
 		} ) ;
+	}else{
+		m_yubekeyNewKey = true ;
+		m_ui->lineEditReEnterPassphrase->setEnabled( false ) ;
+		m_ui->lineEditReEnterPassphrase->clear() ;
+		m_ui->textEditPassphraseToAdd->clear() ;
+		m_ui->textEditPassphraseToAdd->setEchoMode( QLineEdit::Password ) ;
 	}
 }
 
@@ -333,6 +353,9 @@ void luksaddkey::pbAdd( void )
 {
 	DialogMsg msg( this ) ;
 
+	std::function< void() > sendNewKey = [](){} ;
+	std::function< void() > sendExistingKey = [](){} ;
+
 	this->disableAll() ;
 
 	utility::raii raii( [ this ](){ this->enableAll() ; } ) ;
@@ -377,15 +400,18 @@ void luksaddkey::pbAdd( void )
 			return msg.ShowUIOK( tr( "ERROR!" ),tr( "Atleast one required field is empty" ) ) ;
 		}
 	}else{
-		if( NewKey != NewKey_1 ){
+		if( !m_yubekeyNewKey ){
 
-			msg.ShowUIOK( tr( "ERROR!" ),tr( "Keys do not match" ) ) ;
+			if( NewKey != NewKey_1 ){
 
-			m_ui->textEditPassphraseToAdd->clear() ;
-			m_ui->lineEditReEnterPassphrase->clear() ;
-			m_ui->textEditPassphraseToAdd->setFocus() ;
+				msg.ShowUIOK( tr( "ERROR!" ),tr( "Keys do not match" ) ) ;
 
-			return ;
+				m_ui->textEditPassphraseToAdd->clear() ;
+				m_ui->lineEditReEnterPassphrase->clear() ;
+				m_ui->textEditPassphraseToAdd->setFocus() ;
+
+				return ;
+			}
 		}
 	}
 
@@ -396,12 +422,28 @@ void luksaddkey::pbAdd( void )
 		ExistingKey = utility::resolvePath( ExistingKey ).replace( "\"","\"\"\"" ) ;
 		existingPassType = "-u" ;
 	}else{
+		auto k = m_ui->textEditExistingPassphrase->text() ;
+
+		if( m_yubikeyExistingKey ){
+
+			auto m = utility::yubiKey( k ) ;
+
+			if( m.has_value() ){
+
+				k = m.value() ;
+			}else{
+				this->cbExistingKey( 0 ) ;
+				return m_label.show( tr( "Failed To Locate Or Run Yubikey's \"ykchalresp\" Program." ) ) ;
+			}
+		}
+
 		existingPassType = "-u" ;
 		ExistingKey = utility::keyPath() + "-existingKey" ;
 
-		auto k = m_ui->textEditExistingPassphrase->text() ;
+		sendExistingKey = [ = ](){
 
-		utility::keySend( ExistingKey,k ) ;
+			utility::keySend( ExistingKey,k ) ;
+		} ;
 	}
 
 	QString newPassType ;
@@ -411,13 +453,29 @@ void luksaddkey::pbAdd( void )
 		NewKey = utility::resolvePath( NewKey ).replace( "\"","\"\"\"" ) ;
 		newPassType = "-n" ;
 	}else{
+		auto k = m_ui->textEditPassphraseToAdd->text() ;
+
+		if( m_yubekeyNewKey ){
+
+			auto m = utility::yubiKey( k ) ;
+
+			if( m.has_value() ){
+
+				k = m.value() ;
+			}else{
+				this->cbNewKey( 0 ) ;
+				return m_label.show( tr( "Failed To Locate Or Run Yubikey's \"ykchalresp\" Program." ) ) ;
+			}
+		}
+
 		newPassType = "-n" ;
 
 		NewKey = utility::keyPath() + "-newKey" ;
 
-		QString k = m_ui->textEditPassphraseToAdd->text() ;
+		sendNewKey = [ = ](){
 
-		utility::keySend( NewKey,k ) ;
+			utility::keySend( NewKey,k ) ;
+		} ;
 	}
 
 	const QString& a = QString( ZULUCRYPTzuluCrypt ) ;
@@ -453,6 +511,9 @@ void luksaddkey::pbAdd( void )
 
 	raii.cancel() ;
 
+	sendNewKey() ;
+	sendExistingKey() ;
+
 	this->taskFinished( utility::Task::run( exe ).await() ) ;
 }
 
@@ -462,19 +523,16 @@ void luksaddkey::keyAdded()
 
 	auto l = utility::luksEmptySlots( m_volumePath ).await() ;
 
-	decltype( tr( "" ) ) success ;
+	QString success ;
 
 	if( l.isEmpty() ){
 
-		success = tr( "Key added successfully." ) ;
+		m_label.show( tr( "Key added successfully." ) ) ;
 	}else{
 		QString x = tr( "Key added successfully.\n%1 / %2 slots are now in use" ) ;
-		success = x.arg( l.first() ).arg( l.at( 1 ) ) ;
+		m_label.show( x.arg( l.first() ).arg( l.at( 1 ) ) ) ;
 	}
 
-	DialogMsg msg( this ) ;
-	msg.ShowUIOK( tr( "SUCCESS!" ),success ) ;
-
 	this->HideUI() ;
 }
 
@@ -483,28 +541,27 @@ void luksaddkey::taskFinished( const utility::Task& e )
 	m_veraCryptWarning.stopTimer() ;
 
 	m_isWindowClosable = true ;
-	DialogMsg msg( this ) ;
 
 	switch( e.exitCode() ){
 		case 0  : return this->keyAdded() ;
-		case 1  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Presented key does not match any key in the volume" ) ) ;		      	break ;
-		case 2  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not open luks volume" ) ) ;					     	break ;
-		case 3  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Volume is not a luks volume" ) ) ;					     	break ;
-		case 4  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to add a key to a system device,\nonly root user or members of group \"zulucrypt\" can do that\n" ) )	;break ;
-		case 5  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not open volume in write mode" ) ) ;					break ;
-		case 6  : msg.ShowUIOK( tr( "ERROR!" ),tr( "All key slots are occupied, can not add any more keys" ) ) ;		break ;
-		case 7  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Can not get passphrase in silent mode" ) ) ;			   	break ;
-		case 8  : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient memory to hold passphrase" ) ) ;	                      	break ;
-		case 9  : msg.ShowUIOK( tr( "ERROR!" ),tr( "New passphrases do not match" ) ) ;						break ;
-		case 10 : msg.ShowUIOK( tr( "ERROR!" ),tr( "One or more required argument(s) for this operation is missing" ) ) ;      	break ;
-		case 11 : msg.ShowUIOK( tr( "ERROR!" ),tr( "One or both keyfile(s) does not exist" ) ) ;				break ;
-		case 12 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to open key file for reading" ) ) ;			break ;
-		case 13 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Couldnt get enought memory to hold the key file" ) ) ;			break ;
-		case 14 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get a key from a socket" ) ) ;					break ;
-		case 15 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get elevated privilege,check binary permissions" ) ) ;		break ;
-		case 110: msg.ShowUIOK( tr( "ERROR!" ),tr( "Can not find a partition that match presented UUID" ) ) ;			break ;
-		case 113: msg.ShowUIOK( tr( "ERROR!" ),tr( "Device is not a luks device" ) ) ;						break ;
-		default: msg.ShowUIOK( tr( "ERROR!" ),tr( "Error Code: %1\n--\nStdOut: %2\n--\nStdError: %3").arg( QString::number( e.exitCode() ),QString( e.stdError() ),QString( e.stdOut() ) ) ) ;
+		case 1  : m_label.show( tr( "Presented key does not match any key in the volume" ) ) ;		      	break ;
+		case 2  : m_label.show( tr( "Could not open luks volume" ) ) ;					     	break ;
+		case 3  : m_label.show( tr( "Volume is not a luks volume" ) ) ;					     	break ;
+		case 4  : m_label.show( tr( "Insufficient privilege to add a key to a system device,\nonly root user or members of group \"zulucrypt\" can do that\n" ) )	;break ;
+		case 5  : m_label.show( tr( "Could not open volume in write mode" ) ) ;					break ;
+		case 6  : m_label.show( tr( "All key slots are occupied, can not add any more keys" ) ) ;		break ;
+		case 7  : m_label.show( tr( "Can not get passphrase in silent mode" ) ) ;			   	break ;
+		case 8  : m_label.show( tr( "Insufficient memory to hold passphrase" ) ) ;	                      	break ;
+		case 9  : m_label.show( tr( "New passphrases do not match" ) ) ;						break ;
+		case 10 : m_label.show( tr( "One or more required argument(s) for this operation is missing" ) ) ;      	break ;
+		case 11 : m_label.show( tr( "One or both keyfile(s) does not exist" ) ) ;				break ;
+		case 12 : m_label.show( tr( "Insufficient privilege to open key file for reading" ) ) ;			break ;
+		case 13 : m_label.show( tr( "Couldnt get enought memory to hold the key file" ) ) ;			break ;
+		case 14 : m_label.show( tr( "Could not get a key from a socket" ) ) ;					break ;
+		case 15 : m_label.show( tr( "Could not get elevated privilege,check binary permissions" ) ) ;		break ;
+		case 110: m_label.show( tr( "Can not find a partition that match presented UUID" ) ) ;			break ;
+		case 113: m_label.show( tr( "Device is not a luks device" ) ) ;						break ;
+		default:  m_label.show( tr( "Error Code: %1\n--\nStdOut: %2\n--\nStdError: %3").arg( QString::number( e.exitCode() ),QString( e.stdError() ),QString( e.stdOut() ) ) ) ;
 	}
 
 	m_veraCryptWarning.hide() ;
