@@ -72,6 +72,8 @@ passwordDialog::passwordDialog( QTableWidget * table,
 {
 	m_ui->setupUi( this ) ;
 
+	m_label.setOptions( m_ui->veraCryptWarning,m_ui->pbOK ) ;
+
 	m_ui->PassPhraseField->setMaxLength( 32767 ) ;
 
 	m_parent = parent ;
@@ -129,6 +131,8 @@ passwordDialog::passwordDialog( QTableWidget * table,
 
 	m_ui->cbKeyType->addItem( tr( "TrueCrypt/VeraCrypt Keys" ) ) ;
 
+	m_ui->cbKeyType->addItem( tr( "YubiKey Challenge/Response" ) ) ;
+
 	m_ui->pushButtonPlainDmCryptOptions->setMenu( [ this ](){
 
 		auto m = new QMenu( this ) ;
@@ -181,12 +185,12 @@ void passwordDialog::cbVolumeType( int e )
 	if( e == 0 ){
 
 		/*
-		 * LUKS,TrueCrypt
+		 * LUKS,TrueCrypt,BitLocker
 		 */
 		m_ui->labelVolumeProperty->clear() ;
 		m_ui->lineEditVolumeProperty->setEnabled( false ) ;
 
-	}else if( e == 1 ){
+	}else if( e == 1 || e == 2 ){
 
 		/*
 		 * VeraCrypt volume
@@ -194,7 +198,7 @@ void passwordDialog::cbVolumeType( int e )
 		m_ui->labelVolumeProperty->setText( tr( "PIM Value" ) ) ;
 		m_ui->lineEditVolumeProperty->setEnabled( true ) ;
 
-	}else if( e == 2 ){
+	}else if( e == 3 ){
 
 		/*
 		 * PLAIN dm-crypt with offset
@@ -353,7 +357,7 @@ void passwordDialog::mountPointPath( QString path )
 
 void passwordDialog::cbActicated( int e )
 {
-	if( e == passwordDialog::key ){
+	if( e == passwordDialog::key || e == passwordDialog::yubikey ){
 
 		m_ui->checkBoxVisibleKey->setEnabled( true ) ;
 	}else{
@@ -364,6 +368,7 @@ void passwordDialog::cbActicated( int e )
 	switch( e ){
 
 		case passwordDialog::key         : return this->passphraseOption() ;
+		case passwordDialog::yubikey     : return this->passphraseOption() ;
 		case passwordDialog::keyfile     : return this->passphraseFromFileOption() ;
 		case passwordDialog::keyKeyFile  : return this->keyAndKeyFile() ;
 		case passwordDialog::plugin      : return this->pluginOption() ;
@@ -570,7 +575,21 @@ void passwordDialog::buttonOpenClicked( void )
 			this->enableAll() ;
 		}
 	}else{
-		m_key = m_ui->PassPhraseField->text().toLatin1() ;
+		if( m_ui->cbKeyType->currentIndex() == passwordDialog::yubikey ){
+
+			auto m = utility::yubiKey( m_ui->PassPhraseField->text() ) ;
+
+			if( m.has_value() ){
+
+				m_key = m.value() ;
+			}else{
+				DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "Failed To Locate Or Run Yubikey's \"ykchalresp\" Program." ) ) ;
+				return this->enableAll() ;
+			}
+		}else{
+			m_key = m_ui->PassPhraseField->text().toLatin1() ;
+		}
+
 		this->openVolume() ;
 	}
 }
@@ -628,7 +647,8 @@ void passwordDialog::enableAll()
 	m_ui->pushButtonPassPhraseFromFile->setEnabled( true ) ;
 	m_ui->PushButtonVolumePath->setEnabled( true ) ;
 	m_ui->cbKeyType->setEnabled( true ) ;
-	m_ui->checkBoxVisibleKey->setEnabled( m_ui->cbKeyType->currentIndex() == passwordDialog::key ) ;
+
+	auto m = m_ui->cbKeyType->currentIndex() ;
 
 	if( m_open_with_path ){
 
@@ -636,11 +656,13 @@ void passwordDialog::enableAll()
 		m_ui->PushButtonVolumePath->setEnabled( false ) ;
 	}
 
-	if( m_ui->cbKeyType->currentIndex() == passwordDialog::key ){
+	if( m == passwordDialog::key || m == passwordDialog::yubikey ){
 
+		m_ui->checkBoxVisibleKey->setEnabled( true ) ;
 		m_ui->pushButtonPassPhraseFromFile->setEnabled( false ) ;
 		m_ui->PassPhraseField->setEnabled( true ) ;
 	}else{
+		m_ui->checkBoxVisibleKey->setEnabled( false ) ;
 		m_ui->pbKeyOption->setEnabled( true ) ;
 		m_ui->PassPhraseField->setEnabled( false ) ;
 	}
@@ -654,18 +676,14 @@ void passwordDialog::openVolume()
 
 	if( m_point.isEmpty() || m_device.isEmpty() ){
 
-		DialogMsg msg( this ) ;
-
-		msg.ShowUIOK( tr( "ERROR!" ),tr( "Atleast one required field is empty" ) ) ;
+		m_label.show( tr( "Atleast one required field is empty" ) ) ;
 
 		return this->enableAll() ;
 	}
 
 	if( m_point.contains( "/" ) ){
 
-		DialogMsg msg( this ) ;
-
-		msg.ShowUIOK( tr( "ERROR!" ),tr( "\"/\" character is not allowed in mount name field" ) ) ;
+		m_label.show( tr( "\"/\" character is not allowed in mount name field" ) ) ;
 
 		m_ui->OpenVolumePath->setFocus() ;
 
@@ -691,16 +709,16 @@ void passwordDialog::openVolume()
 
 		if( m_key.isEmpty() ){
 
-			DialogMsg msg( this ) ;
-
-			msg.ShowUIOK( tr( "ERROR!" ),tr( "Atleast one required field is empty" ) ) ;
+			m_label.show( tr( "Atleast one required field is empty" ) ) ;
 
 			return this->enableAll() ;
 		}else{
 			passtype = "-f" ;
 			keyPath = utility::resolvePath( m_key ).replace( "\"","\"\"\"" ) ;
 		}
-	}else if( keySource == passwordDialog::key || keySource == passwordDialog::keyKeyFile ){
+	}else if( keySource == passwordDialog::key ||
+		  keySource == passwordDialog::keyKeyFile ||
+		  keySource == passwordDialog::yubikey ){
 
 		passtype = "-f" ;
 		keyPath = utility::keyPath() ;
@@ -711,9 +729,7 @@ void passwordDialog::openVolume()
 
 		if( m_key.isEmpty() ){
 
-			DialogMsg msg( this ) ;
-
-			msg.ShowUIOK( tr( "ERROR!" ),tr( "Atleast one required field is empty" ) ) ;
+			m_label.show( tr( "Atleast one required field is empty" ) ) ;
 
 			return this->enableAll() ;
 		}else{
@@ -749,9 +765,7 @@ void passwordDialog::openVolume()
 
 				if( q.size() < 2 ){
 
-					DialogMsg msg( this ) ;
-
-					msg.ShowUIOK( tr( "ERROR!" ),tr( "Volume is not a LUKS volume" ) ) ;
+					m_label.show( tr( "Volume is not a LUKS volume" ) ) ;
 
 					m_ui->OpenVolumePath->setFocus() ;
 
@@ -770,9 +784,7 @@ void passwordDialog::openVolume()
 
 						this->sendKey( keyPath ) ;
 					}else{
-						DialogMsg msg( this ) ;
-
-						msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to get a key from the network" ) ) ;
+						m_label.show( tr( "Failed to get a key from the network" ) ) ;
 
 						m_ui->OpenVolumePath->setFocus() ;
 						return this->enableAll() ;
@@ -802,6 +814,16 @@ void passwordDialog::openVolume()
 	const QString& e = passtype ;
 	const QString& f = keyPath ;
 
+	if( !b.startsWith( "/dev/" ) ){
+
+		auto s = utility::loopDevicePath( b ) ;
+
+		if( !s.isEmpty() ){
+
+			b = s ;
+		}
+	}
+
 	auto exe = QString( "%1 -o -d \"%2\" -m \"%3\" -e %4 %5 \"%6\"" ).arg( a,b,c,d,e,f ) ;
 
 	if( !m_keyFiles.isEmpty() ){
@@ -815,7 +837,7 @@ void passwordDialog::openVolume()
 		}
 	}
 
-	if( m_veraCryptVolume ){
+	if( m_ui->cbVolumeType->currentIndex() == 1 ){
 
 		auto e = m_ui->lineEditVolumeProperty->text() ;
 
@@ -825,17 +847,27 @@ void passwordDialog::openVolume()
 		}else {
 			exe += " -t vcrypt." + e ;
 		}
-	}else{
-		if( m_ui->cbVolumeType->currentIndex() == 2 ){
 
-			auto e = m_ui->lineEditVolumeProperty->text() ;
+	}else if( m_ui->cbVolumeType->currentIndex() == 2 ){
 
-			if( e.isEmpty() ){
+		auto e = m_ui->lineEditVolumeProperty->text() ;
 
-				exe += " -t " + m_plainDmCryptProperty + ".0" ;
-			}else {
-				exe += " -t " + m_plainDmCryptProperty + "." + e ;
-			}
+		if( e.isEmpty() ){
+
+			exe += " -t vcrypt-sys" ;
+		}else {
+			exe += " -t vcrypt-sys." + e ;
+		}
+
+	}else if( m_ui->cbVolumeType->currentIndex() == 3 ){
+
+		auto e = m_ui->lineEditVolumeProperty->text() ;
+
+		if( e.isEmpty() ){
+
+			exe += " -t " + m_plainDmCryptProperty + ".0" ;
+		}else {
+			exe += " -t " + m_plainDmCryptProperty + "." + e ;
 		}
 	}
 
@@ -879,35 +911,34 @@ void passwordDialog::failed( const utility::Task& e )
 		return this->enableAll() ;
 	}
 
-	DialogMsg msg( this ) ;
 
 	switch ( r ){
-		case 0 : ;
-		case 1 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount ntfs/exfat file system using ntfs-3g,is ntfs-3g/exfat package installed?" ) ) ; break ;
-		case 2 : msg.ShowUIOK( tr( "ERROR!" ),tr( "There seem to be an open volume accociated with given address" ) ) ;				break ;
-		case 3 : msg.ShowUIOK( tr( "ERROR!" ),tr( "No file or device exist on given path" ) ) ; 						break ;
-		case 4 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Volume could not be opened with the presented key" ) ) ;					break ;
-		case 5 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to mount the device with given options" ) ) ;				break ;
-		case 6 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to open device in read write mode or device does not exist" ) ) ;	break ;
-		case 7 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Only root user can perform this operation" ) ) ;						break ;
-		case 8 : msg.ShowUIOK( tr( "ERROR!" ),tr( "-O and -m options can not be used together" ) ) ;						break ;
-		case 9 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not create mount point, invalid path or path already taken" ) ) ;			break ;
-		case 10: msg.ShowUIOK( tr( "ERROR!" ),tr( "Shared mount point path already taken" ) ) ;							break ;
-		case 11: msg.ShowUIOK( tr( "ERROR!" ),tr( "There seem to be an opened mapper associated with the device" ) ) ;				break ;
-		case 12: msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get a passphrase from the module" ) ) ;						break ;
-		case 13: msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get passphrase in silent mode" ) ) ;						break ;
-		case 14: msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient memory to hold passphrase" ) ) ;							break ;
-		case 15: msg.ShowUIOK( tr( "ERROR!" ),tr( "One or more required argument(s) for this operation is missing" ) ) ;				break ;
-		case 16: msg.ShowUIOK( tr( "ERROR!" ),tr( "Invalid path to key file" ) ) ;								break ;
-		case 17: msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get enought memory to hold the key file" ) ) ;					break ;
-		case 18: msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to open key file for reading" ) ) ;					break ;
-		case 19: msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not get a passphrase through a local socket" ) ) ;					break ;
-		case 20: msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount a filesystem:invalid/unsupported mount option or unsupported file system encountered" ) ) ;	break ;
-		case 21: msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not create a lock on /etc/mtab" ) ) ;							break ;
-		case 22: msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to open a system volume.\n\nConsult menu->help->permission for more informaion\n" ) ) ;					break ;
-		case 113:msg.ShowUIOK( tr( "ERROR!" ),tr( "A non supported device encountered,device is missing or permission denied\n\
+		case 0 : return ;
+		case 1 : m_label.show( tr( "Failed to mount ntfs/exfat file system using ntfs-3g,is ntfs-3g/exfat package installed?" ) ) ; break ;
+		case 2 : m_label.show( tr( "There seem to be an open volume accociated with given address" ) ) ;				break ;
+		case 3 : m_label.show( tr( "No file or device exist on given path" ) ) ; 						break ;
+		case 4 : m_label.show( tr( "Volume could not be opened with the presented key" ) ) ;					break ;
+		case 5 : m_label.show( tr( "Insufficient privilege to mount the device with given options" ) ) ;				break ;
+		case 6 : m_label.show( tr( "Insufficient privilege to open device in read write mode or device does not exist" ) ) ;	break ;
+		case 7 : m_label.show( tr( "Only root user can perform this operation" ) ) ;						break ;
+		case 8 : m_label.show( tr( "-O and -m options can not be used together" ) ) ;						break ;
+		case 9 : m_label.show( tr( "Could not create mount point, invalid path or path already taken" ) ) ;			break ;
+		case 10: m_label.show( tr( "Shared mount point path already taken" ) ) ;							break ;
+		case 11: m_label.show( tr( "There seem to be an opened mapper associated with the device" ) ) ;				break ;
+		case 12: m_label.show( tr( "Could not get a passphrase from the module" ) ) ;						break ;
+		case 13: m_label.show( tr( "Could not get passphrase in silent mode" ) ) ;						break ;
+		case 14: m_label.show( tr( "Insufficient memory to hold passphrase" ) ) ;							break ;
+		case 15: m_label.show( tr( "One or more required argument(s) for this operation is missing" ) ) ;				break ;
+		case 16: m_label.show( tr( "Invalid path to key file" ) ) ;								break ;
+		case 17: m_label.show( tr( "Could not get enought memory to hold the key file" ) ) ;					break ;
+		case 18: m_label.show( tr( "Insufficient privilege to open key file for reading" ) ) ;					break ;
+		case 19: m_label.show( tr( "Could not get a passphrase through a local socket" ) ) ;					break ;
+		case 20: m_label.show( tr( "Failed to mount a filesystem:invalid/unsupported mount option or unsupported file system encountered" ) ) ;	break ;
+		case 21: m_label.show( tr( "Could not create a lock on /etc/mtab" ) ) ;							break ;
+		case 22: m_label.show( tr( "Insufficient privilege to open a system volume.\n\nConsult menu->help->permission for more informaion\n" ) ) ;					break ;
+		case 113:m_label.show( tr( "A non supported device encountered,device is missing or permission denied\n\
 Possible reasons for getting the error are:\n1.Device path is invalid.\n2.The device has LVM or MDRAID signature" ) ) ;					break ;
-		default: msg.ShowUIOK( tr( "ERROR!" ),tr( "Error Code: %1\n--\nStdOut: %2\n--\nStdError: %3").arg( QString::number( e.exitCode() ),QString( e.stdError() ),QString( e.stdOut() ) ) ) ;
+		default: m_label.show( tr( "Error Code: %1\n--\nStdOut: %2\n--\nStdError: %3").arg( QString::number( e.exitCode() ),QString( e.stdError() ),QString( e.stdOut() ) ) ) ;
 	}
 
 	utility::debug() << e.stdOut() << "--" << e.stdError() ;
@@ -916,7 +947,7 @@ Possible reasons for getting the error are:\n1.Device path is invalid.\n2.The de
 
 	auto index = m_ui->cbKeyType->currentIndex() ;
 
-	if( utility:: clearPassword() && index == passwordDialog::key ){
+	if( utility:: clearPassword() && ( index == passwordDialog::key || index == passwordDialog::yubikey ) ){
 
 		m_ui->PassPhraseField->clear() ;
 		m_ui->PassPhraseField->setFocus() ;
@@ -924,7 +955,7 @@ Possible reasons for getting the error are:\n1.Device path is invalid.\n2.The de
 
 	if( r == 4 ){
 
-		if( index == passwordDialog::key ){
+		if( index == passwordDialog::key || index == passwordDialog::yubikey ){
 
 			m_ui->PassPhraseField->setFocus() ;
 
