@@ -105,6 +105,7 @@ static QSettings * _settings ;
 
 static debugWindow * _debugWindow ;
 
+static QWidget * _mainWindow ;
 
 static void _post_backend_cmd( const QString& b )
 {
@@ -119,6 +120,17 @@ void utility::setDebugWindow( debugWindow * w )
 	_debugWindow = w ;
 }
 
+void utility::mainWindowWidget( QWidget * e )
+{
+	_mainWindow = e ;
+}
+
+QWidget * utility::mainWindowWidget()
+{
+	return _mainWindow ;
+}
+
+
 std::unique_ptr< utility::RandomDataSource > utility::RandomDataSource::get( utility::RandomDataSource::types type )
 {
 	if( type == utility::RandomDataSource::types::urandom ){
@@ -2876,3 +2888,95 @@ QByteArray utility::CRandDataSource::getData( qint64 size )
 
 	return data ;
 }
+
+QString utility::loopDevicePath( const QString& e )
+{
+	const auto s = QDir( "/sys/block" ).entryList() ;
+
+	QFile file ;
+
+	for( const auto& it : s ){
+
+		if( it.startsWith( "loop" ) ){
+
+			QString m = "/sys/block/" + it + "/loop/backing_file" ;
+
+			if( utility::pathExists( m ) ){
+
+				file.setFileName( m ) ;
+
+				file.open( QIODevice::ReadOnly ) ;
+
+				QString s = file.readAll() ;
+
+				file.close() ;
+
+				if( s.startsWith( e ) ){
+
+					return "/dev/" + it ;
+				}
+			}
+		}
+	}
+
+	return QString() ;
+}
+
+static bool _removeYkchalrespNewLineCharacter()
+{
+	if( !_settings->contains( "RemoveYkchalrespNewLineCharacter" ) ){
+
+		_settings->setValue( "RemoveYkchalrespNewLineCharacter",true ) ;
+	}
+
+	return _settings->value( "RemoveYkchalrespNewLineCharacter" ).toBool() ;
+}
+
+static QString _ykchalrespArguments()
+{
+	if( !_settings->contains( "YkchalrespArguments" ) ){
+
+		_settings->setValue( "YkchalrespArguments","-2 -i -" ) ;
+	}
+
+	return _settings->value( "YkchalrespArguments" ).toString() ;
+}
+
+static QString _ykchalresp_path()
+{
+	static QString m = utility::executableFullPath( "ykchalresp" ) ;
+	return m ;
+}
+
+
+utility::result<QByteArray> utility::yubiKey( const QString& challenge )
+{
+	QString exe = _ykchalresp_path() ;
+
+	if( !exe.isEmpty() ){
+
+		exe = exe + " " + _ykchalrespArguments() ;
+
+		//_post_backend_cmd( exe ) ;
+
+		auto s = ::Task::process::run( exe,challenge.toLatin1() ).await() ;
+
+		if( s.success() ){
+
+			auto m = s.std_out() ;
+
+			if( _removeYkchalrespNewLineCharacter() ){
+
+				m.replace( "\n","" ) ;
+			}
+
+			return m ;
+		}else{
+			std::cout << "Failed to get a responce from ykchalresp" << std::endl ;
+			std::cout << "StdOUt:" << s.std_out().constData() << std::endl ;
+			std::cout << "StdError:" << s.std_error().constData() << std::endl ;
+		}
+	}
+
+	return {} ;
+}
