@@ -37,6 +37,7 @@
 #include <QVector>
 #include <QSystemTrayIcon>
 #include <QAction>
+#include <QEventLoop>
 #include <QIcon>
 #include <QByteArray>
 #include <QEvent>
@@ -60,6 +61,8 @@
 
 #include <QObject>
 #include <QLabel>
+#include <QPushButton>
+#include <QLabel>
 
 #include <poll.h>
 #include <fcntl.h>
@@ -137,6 +140,43 @@ public:
 		bool m_run = true ;
 		std::function< void() > m_function ;
 	};
+
+	class label
+	{
+	public:
+		void setOptions( QLabel * l,QPushButton * b )
+		{
+			m_label = l ;
+			m_pushButton = b ;
+
+			m_label->setVisible( false ) ;
+			m_pushButton->setVisible( false ) ;
+
+			QObject::connect( m_pushButton,&QPushButton::clicked,[ this ](){
+
+				this->hide() ;
+				m_eventLoop.exit() ;
+			} ) ;
+		}
+		void show( const QString& e )
+		{
+			m_label->setText( e ) ;
+			m_label->setVisible( true ) ;
+			m_pushButton->setVisible( true ) ;
+			m_pushButton->setFocus() ;
+			m_eventLoop.exec() ;
+		}
+		void hide()
+		{
+			m_label->clear() ;
+			m_label->setVisible( false ) ;
+			m_pushButton->setVisible( false ) ;
+		}
+	private:
+		QLabel * m_label ;
+		QPushButton * m_pushButton ;
+		QEventLoop m_eventLoop ;
+	};
 }
 namespace utility
 {
@@ -380,6 +420,7 @@ namespace utility
 	::Task::future< utility::result< bool > >& backendIsGreaterOrEqualTo( const QString& backend,
 									      const QString& version ) ;
 
+	utility::result< QByteArray > yubiKey( const QString& ) ;
 	bool useDmCryptForRandomData() ;
 	void setDefaultEnvironment() ;
 	QString passwordSocketPath() ;
@@ -400,11 +441,14 @@ namespace utility
 	void dropPrivileges( int = -1 ) ;
 	void setDebugWindow( debugWindow * w ) ;
 	QString fileManager( void ) ;
+	QString loopDevicePath( const QString& ) ;
 	void setFileManager( const QString& ) ;
 	QString failedToStartzuluPolkit() ;
 	QString prettyfySpaceUsage( quint64 ) ;
 	QString resolvePath( const QString& ) ;
 	QString hashPath( const QByteArray& ) ;
+	QWidget * mainWindowWidget() ;
+	void mainWindowWidget( QWidget * ) ;
 	QString cryptMapperPath( void ) ;
 	QString mapperPath( const QString&,const QString& component = QString() ) ;
 	QString getVolumeID( const QString&,bool = false ) ;
