@@ -51,6 +51,8 @@ mountPartition::mountPartition( QWidget * parent,QTableWidget * table,std::funct
 	m_ui->checkBoxShareMountPoint->setToolTip( utility::shareMountPointToolTip() ) ;
 	m_table = table ;
 
+	m_Label.setOptions( m_ui->label,m_ui->pushButton ) ;
+
 	this->setFixedSize( this->size() ) ;
 	this->setWindowFlags( Qt::Window | Qt::Dialog ) ;
 	this->setFont( parent->font() ) ;
@@ -153,21 +155,21 @@ bool mountPartition::errorNotFound( int r )
 
 	switch ( r ){
 		case 0 : break ;
-		case 112 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not resolve path to device or device could not be opened in read write mode" ) ) ; break ;
-		case 100 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insuffienct privileges to mount the volume with given mount options" ) ) ;				break ;
-		case 102 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Device already mounted" ) ) ; 						break ;
+		case 112 : m_Label.show( tr( "Could not resolve path to device or device could not be opened in read write mode" ) ) ;				break ;
+		case 100 : m_Label.show( tr( "Insuffienct privileges to mount the volume with given mount options" ) ) ;					break ;
+		case 102 : m_Label.show( tr( "Device already mounted" ) ) ;											break ;
 		case 103 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Insuffienct privilege to manage a system volume.\nnecessary privileges can be acquired by:\n\
-1. Adding an entry for the volume in fstab with \"user\" mount option\n\2. Add yourself to \"zulumount\" group" ) ) ;					break ;
-		case 104 : msg.ShowUIOK( tr( "ERROR!" ),tr( "\"/etc/fstab\" entry for this volume requires it to be mounted read only" ) ) ;		break ;
-		case 113 : msg.ShowUIOK( tr( "ERROR!" ),tr( "\"/etc/fstab\" entry for this volume is malformed" ) ) ;					break ;
-		case 105 : msg.ShowUIOK( tr( "ERROR!" ),tr( "\"/etc/fstab\" entry for this volume does not allow you to mount it" ) ) ;			break ;
-		case 106 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Could not create mount point path,path already taken" ) ) ;				break ;
-		case 114 : msg.ShowUIOK( tr( "ERROR!" ),tr( "Shared mount point path aleady taken" ) ) ;						break ;
-		case 108: msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount a filesystem:invalid/unsupported mount option or unsupported file system encountered" ) ) ;	break ;
-		case 109: msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount ntfs/exfat file system using ntfs-3g,is ntfs-3g/exfat package installed?" ) )	      ;	break ;
-		case 110: msg.ShowUIOK( tr( "ERROR!" ),tr( "Mount failed,no or unrecognized file system" ) ) ;						break ;
+1. Adding an entry for the volume in fstab with \"user\" mount option\n\2. Add yourself to \"zulumount\" group" ) ) ;						break ;
+		case 104 : m_Label.show( tr( "\"/etc/fstab\" entry for this volume requires it to be mounted read only" ) ) ;					break ;
+		case 113 : m_Label.show( tr( "\"/etc/fstab\" entry for this volume is malformed" ) ) ;								break ;
+		case 105 : m_Label.show( tr( "\"/etc/fstab\" entry for this volume does not allow you to mount it" ) ) ;					break ;
+		case 106 : m_Label.show( tr( "Could not create mount point path,path already taken" ) ) ;							break ;
+		case 114 : m_Label.show( tr( "Shared mount point path aleady taken" ) ) ;									break ;
+		case 108 : m_Label.show( tr( "Failed to mount a filesystem:invalid/unsupported mount option or unsupported file system encountered" ) ) ;	break ;
+		case 109 : m_Label.show( tr( "Failed to mount ntfs/exfat file system using ntfs-3g,is ntfs-3g/exfat package installed?" ) )	      ;		break ;
+		case 110 : m_Label.show( tr( "Mount failed,no or unrecognized file system" ) ) ;								break ;
 		case 111: msg.ShowUIOK( tr( "ERROR!" ),tr( "Mount failed,could not get a lock on /etc/mtab~" ) ) ;						break ;
-		case 115: msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount the partition" ) ) ;						break ;
+		case 115: msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to mount the partition" ) ) ;								break ;
 		default: return true ;
 	}
 
@@ -196,6 +198,17 @@ void mountPartition::pbMount()
 	QString exe = zuluMountPath ;
 
 	auto volume = m_path ;
+
+	if( !volume.startsWith( "/dev/" ) ){
+
+		auto m = utility::loopDevicePath( volume ) ;
+
+		if( !m.isEmpty() ){
+
+			volume = m ;
+		}
+	}
+
 	volume.replace( "\"","\"\"\"" ) ;
 
 	if( m_ui->checkBoxShareMountPoint->isChecked() ){
