@@ -58,6 +58,7 @@
 #include "task.hpp"
 #include "../zuluCrypt-gui/checkforupdates.h"
 #include "../zuluCrypt-gui/favorites.h"
+#include "bin_path.h"
 
 #include <memory>
 
@@ -73,6 +74,8 @@ zuluMount::zuluMount( QWidget * parent ) :
 	m_checkForUpdates( this ),
 	m_signalHandler( this )
 {
+	utility::mainWindowWidget( this ) ;
+
 	m_signalHandler.setAction( [ this ]( systemSignalHandler::signal s ){
 
 		Q_UNUSED( s ) ;
@@ -231,6 +234,20 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 		return ac ;
 	}() ) ;
 
+	trayMenu->addAction( [ this ](){
+
+		auto ac = new QAction( tr( "Clear Dead Mount Points" ),this ) ;
+
+		m_actionPair.append( { ac,"Clear Dead Mount Points" } ) ;
+
+		connect( ac,&QAction::triggered,[](){
+
+			utility::Task::run( zuluMountPath" --clear-dead-mount-points" ).await() ;
+		} ) ;
+
+		return ac ;
+	}() ) ;
+
 	trayMenu->addAction( [ this ](){
 
 		auto ac = new QAction( tr( "Unmount All" ),this ) ;
